//------------------------------------------------------------------------------
//
// File:    xlifeModuleAppl.h
//
// Purpose: Module for Test application.
//   
// (c) Plekhanov Andrey
//
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//------------------------------------------------------------------------------

#ifndef XLIFE_MODULE_APPL_H
#define XLIFE_MODULE_APPL_H

#include "apsmainmodule.h"
#include "xlifeConfig.h"

namespace aps {

  namespace xlifeappl {

class XLifeSubModule;
class XLifeConfigReadSubModule;

using apslib::APSMainModule;

enum {
  XLIFE_APPL_NO_ERR = 0,
  XLIFE_APPLICATION,
  XLIFE_APPL_MODULE,
  XLIFE_APPL_START,
  XLIFE_APPL_FINISH,
  XLIFE_APPL_SHOW_PARAM
};

enum {
  XLIFE_APPL_PARAM_DEFAULT = 0,
  XLIFE_APPL_PARAM_MAIN_CONFIG,
  XLIFE_APPL_PARAM_CALCULATED
};

//======================= XLifeModuleAppl ==========================

class XLifeModuleAppl : public APSMainModule
{
  private:

    XLifeSubModule           * pXLifeSubModule;
    XLifeConfigReadSubModule * pXLifeConfigReadSubModule;

    std::string ProjectFilePath;

    int BorderField;
    int Delta;
    int Width;
    int Height;
    int InitRandom;

    int OriginBorderField;
    int OriginDelta;
    int OriginWidth;
    int OriginHeight;
    int OriginInitRandom;

  public:

    XLifeModuleAppl( const std::string & aProjectFilePath,
                     const std::string & aModuleName,
                     const int X_Version,
                     const int Y_Version,
                     const int Z_Version,
                     const int FirstCopyRightYear,
                     const int CopyRightYear,
                     const apslib::APS_PROGRAM_VERSION_TYPE VersionType = apslib::APS_NORMAL_VERSION );

    virtual ~XLifeModuleAppl( void );

    XLifeSubModule * GetXLifeSubModulePtr( void ) const
      { return( pXLifeSubModule ); }

    virtual const std::string GetMessageText( const int MsgNumber ) const;

    int GetBorderField( void ) const
      { return( BorderField ); }

    int GetDelta( void ) const
      { return( Delta ); }

    int GetWidth( void ) const
      { return( Width ); }

    int GetHeight( void ) const
      { return( Height ); }

    int GetInitRandom( void ) const
      { return( InitRandom ); }

    bool IfWidth( void ) const;

    bool IfHeight( void ) const;

    bool IfInitRandom( void ) const;

    void PrintParameters( void ) const;

    virtual int Run( void );
};

}}

#endif

//---------------------------- End of file ---------------------------



