//------------------------------------------------------------------------------
//
// File:    xlifeProg.cc
//
// Purpose: Life game for XWindow
//   
// (c) Plekhanov Andrey
//
// Initial version 0.1 03.01.2005
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//------------------------------------------------------------------------------

#include <iostream>
#include <ctime>
#include <cmath>
#include <unistd.h>

#define XK_LATIN1 1
#define XK_MISCELLANY 1
#include <X11/keysymdef.h>
#include <X11/Xutil.h>

#include "xlifeProg.h"
#include "xlifeModule.h"
#include "apsrandom.h"

namespace aps {

  namespace xlifeappl {

OnePoint :: OnePoint( POINT_VALUE * apValue )
{
  int i;

  pValue = apValue;
  *pValue = 0;

  for( i = 0; i < 9; i++ ) {
    ValueArray[ i ] = 0;
  }
}

OnePoint :: ~OnePoint( void )
{
}

void OnePoint :: SetValueArray( const unsigned int Number, const unsigned int Width,
                                const unsigned int Height, POINT_VALUE * pField )
{
  unsigned int i;
  unsigned int j;

  i = Number / Width;
  j = Number % Width;

  ValueArray[ 4 ] = &pField[ Number ];

  if( i > 0 ) {
    ValueArray[ 1 ] = &pField[ ( i - 1 ) * Width + j ];

    if( j > 0 ) {
      ValueArray[ 0 ] = &pField[ ( i - 1 ) * Width + j - 1 ];
    }
    else { // i > 0 j == 0
      ValueArray[ 0 ] = &pField[ i * Width - 1 ];
    }

    if( j < Width - 1 ) {
      ValueArray[ 2 ] = &pField[ ( i - 1 ) * Width + j + 1 ];
    }
    else { // i > 0 j == Width - 1
      ValueArray[ 2 ] = &pField[ ( i - 1 ) * Width ];
    }
  }
  else { // i == 0
    ValueArray[ 1 ] = &pField[ ( Height - 1 ) * Width + j ];

    if( j > 0 ) {
      ValueArray[ 0 ] = &pField[ ( Height - 1 ) * Width + j - 1 ];
    }
    else { // i == 0 j == 0
      ValueArray[ 0 ] = &pField[ Height * Width - 1 ];
    }

    if( j < Width - 1 ) {
      ValueArray[ 2 ] = &pField[ ( Height - 1 ) * Width + j + 1 ];
    }
    else { // i == 0 j == Width - 1
      ValueArray[ 2 ] = &pField[ ( Height - 1 ) * Width ];
    }
  }

  if( i < Height - 1 ) {
    ValueArray[ 7 ] = &pField[ ( i + 1 ) * Width + j ];

    if( j > 0 ) {
      ValueArray[ 6 ] = &pField[ ( i + 1 ) * Width + j - 1 ];
    }
    else { // i < Height - 1 j == 0
      ValueArray[ 6 ] = &pField[ ( i + 2 ) * Width - 1 ];
    }

    if( j < Width - 1 ) {
      ValueArray[ 8 ] = &pField[ ( i + 1 ) * Width + j + 1 ];
    }
    else { // i < Height - 1 j == Width - 1
      ValueArray[ 8 ] = &pField[ ( i + 1 ) * Width ];
    }
  }
  else { // i == Height - 1
    ValueArray[ 7 ] = &pField[ j ];

    if( j > 0 ) {
      ValueArray[ 6 ] = &pField[ j - 1 ];
    }
    else { // i == Height - 1 j == 0
      ValueArray[ 6 ] = &pField[ Width - 1 ];
    }

    if( j < Width - 1 ) {
      ValueArray[ 8 ] = &pField[ j + 1 ];
    }
    else { // i == Height - 1 j == Width - 1
      ValueArray[ 8 ] = &pField[ 0 ];
    }
  }

  if( j > 0 ) {
    ValueArray[ 3 ] = &pField[ i * Width + j - 1 ];
  }
  else { // j == 0
    ValueArray[ 3 ] = &pField[ ( i + 1 ) * Width - 1 ];
  }

  if( j < Width - 1 ) {
    ValueArray[ 5 ] = &pField[ i * Width + j + 1 ];
  }
  else { // j == Width - 1
    ValueArray[ 5 ] = &pField[ i * Width ];
  }

  for( int k = 0; k < 9; k++ ) {
    if( !ValueArray[ k ] ) {
      printf("ERROR: ValueArray == 0 k = %d i = %d j = %d\n", k, i, j );
    }
  }
}

XLifeField :: XLifeField( const unsigned int aWidth, const unsigned int aHeight )
{
  unsigned int i;
  POINT_VALUE  Value;

  Width  = aWidth;
  Height = aHeight;
  PointsNumber = aWidth * aHeight;
  page   = 0;

  pField1 = new POINT_VALUE[ PointsNumber ];
  pField2 = new POINT_VALUE[ PointsNumber ];

  ppPoint1 = new OnePoint * [ PointsNumber ];
  ppPoint2 = new OnePoint * [ PointsNumber ];

  for( i = 0; i < PointsNumber; i++ ) {
    ppPoint1[ i ] = new OnePoint( &pField1[ i ] );
    ppPoint2[ i ] = new OnePoint( &pField2[ i ] );

    ppPoint1[ i ]->SetValueArray( i, aWidth, aHeight, pField2 );
    ppPoint2[ i ]->SetValueArray( i, aWidth, aHeight, pField1 );
  }

  bzero( pField1, sizeof( POINT_VALUE ) * PointsNumber );
  bzero( pField2, sizeof( POINT_VALUE ) * PointsNumber );

  for( Value = 0; Value < 512; Value++ ) {
    Table[ Value ] = 0;

    i = Sum( Value );

    if( i == 2 ) {
      if( Value & POINT5 ) {
        Table[ Value ] = 1;
      }
    }

    if( i == 3 ) {
      Table[ Value ] = 1;
    }
  }
}

XLifeField :: ~XLifeField( void )
{
  unsigned int i;

  for( i = 0; i < PointsNumber; i++ ) {
    delete ppPoint1[ i ];
    delete ppPoint2[ i ];
  }

  delete ppPoint1;
  delete ppPoint2;

  delete pField1;
  delete pField2;
}

int XLifeField :: Sum( const POINT_VALUE Value ) const
{
  int Summa;

  Summa = 0;

  if( Value & POINT1 ) {
    Summa++;
  }

  if( Value & POINT2 ) {
    Summa++;
  }

  if( Value & POINT3 ) {
    Summa++;
  }

  if( Value & POINT4 ) {
    Summa++;
  }

  if( Value & POINT6 ) {
    Summa++;
  }

  if( Value & POINT7 ) {
    Summa++;
  }

  if( Value & POINT8 ) {
    Summa++;
  }

  if( Value & POINT9 ) {
    Summa++;
  }

  return( Summa );
}

void XLifeField :: Step( void )
{
  unsigned int i;  

  if( page ) {
    bzero( pField1, sizeof( POINT_VALUE ) * PointsNumber );

    for( i = 0; i < PointsNumber; i++ ) {
      if( Table[ pField2[ i ] ] ) {
        ppPoint2[ i ]->Set();
      }
    }
  }
  else {
    bzero( pField2, sizeof( POINT_VALUE ) * PointsNumber );

    for( i = 0; i < PointsNumber; i++ ) {
      if( Table[ pField1[ i ] ] ) {
        ppPoint1[ i ]->Set();
      }
    }
  }

  page = !page;
}

unsigned int XLifeField :: Fill( XPoint * pPoints ) const
{
  unsigned int i;
  unsigned int Number;

  Number = 0;

  if( page ) {
    for( i = 0; i < PointsNumber; i++ ) {
      if( pField2[ i ] & POINT5 ) {
        pPoints[ Number ].x = i % Width;
        pPoints[ Number ].y = i / Width;

        Number++;
      }
    }
  }
  else {
    for( i = 0; i < PointsNumber; i++ ) {
      if( pField1[ i ] & POINT5 ) {
        pPoints[ Number ].x = i % Width;
        pPoints[ Number ].y = i / Width;

        Number++;
      }
    }
  }

  return( Number );
}

void XLifeField :: InitRec( const unsigned int StartX, const unsigned int StartY,
                            const unsigned int StartWidth, const unsigned int StartHeight )
{
  unsigned int i;
  unsigned int j;

  for( i = StartX; i < StartWidth + StartX; i++ ) {
    for( j = StartY; j < StartHeight + StartY; j++ ) {
      if( !apslib::GetRandomNumber( 5 ) ) {
        ppPoint2[ j * Width + i ]->Set();
      }
    }
  }
}

void XLifeField :: InitNorm( const unsigned int StartX, const unsigned int StartY,
                             const double Sigma, const double Number )
{
  unsigned int i;
  unsigned int j;

  for( i = 0; i < Width; i++ ) {
    for( j = 0; j < Height; j++ ) {
      if( !apslib::GetRandomNumber( (int)(Number + 5 * sin( Sigma * sqrt( ( i - StartX ) * ( i - StartX ) + ( j - StartY ) * ( j - StartY ) ) ) ) ) ) {
        ppPoint2[ j * Width + i ]->Set();
      }
    }
  }
}

void XLifeField :: Init( const bool IfInitRandom, const unsigned int InitRandom )
{
  /*InitRec( ( Width - 100 ) / 4, ( Height - 100 ) / 4, 100, 100 );
  InitRec( ( Width - 100 ) / 4, 3 * ( Height - 100 ) / 4, 100, 100 );
  InitRec( 3 * ( Width - 100 ) / 4, ( Height - 100 ) / 4, 100, 100 );
  InitRec( 3 * ( Width - 100 ) / 4, 3 * ( Height - 100 ) / 4, 100, 100 );

  InitRec( ( Width - 200 ) / 2, ( Height - 200 ) / 2, 200, 200 );*/

  /*InitRec( ( Width - 100 ) / 4, ( Height - 100 ) / 4, 100, 100 );
  InitRec( ( Width - 100 ) / 4, 2 * ( Height - 100 ) / 4, 100, 100 );
  InitRec( ( Width - 100 ) / 4, 3 * ( Height - 100 ) / 4, 100, 100 );

  InitRec( 2 * ( Width - 100 ) / 4, ( Height - 100 ) / 4, 100, 100 );
  //InitRec( 2 * ( Width - 100 ) / 4, 2 * ( Height - 100 ) / 4, 100, 100 );
  InitRec( 2 * ( Width - 100 ) / 4, 3 * ( Height - 100 ) / 4, 100, 100 );

  InitRec( 3 * ( Width - 100 ) / 4, ( Height - 100 ) / 4, 100, 100 );
  InitRec( 3 * ( Width - 100 ) / 4, 2 * ( Height - 100 ) / 4, 100, 100 );
  InitRec( 3 * ( Width - 100 ) / 4, 3 * ( Height - 100 ) / 4, 100, 100 );*/

  if( IfInitRandom ) {
    apslib::SetCurrent( InitRandom );
  }
  else {
    apslib::SetCurrent( time( 0 ) );
  }

  InitRec( 0, 0, Width, Height );

  /*InitRec( 0, 0, 100, 100 );
  InitRec( Width - 100, 0, 100, 100 );
  InitRec( 0, Height - 100, 100, 100 );
  InitRec( Width - 100, Height - 100, 100, 100 );*/

  /*InitNorm( ( Width - 100 ) / 4, ( Height - 100 ) / 4, 0.5, 1.5 );
  InitNorm( ( Width - 100 ) / 4, 2 * ( Height - 100 ) / 4, 0.5, 1.5 );
  InitNorm( ( Width - 100 ) / 4, 3 * ( Height - 100 ) / 4, 0.5, 1.5 );

  InitNorm( 2 * ( Width - 100 ) / 4, ( Height - 100 ) / 4, 0.5, 1.5 );
  InitNorm( 2 * ( Width - 100 ) / 4, 2 * ( Height - 100 ) / 4, 0.5, 1.5 );
  InitNorm( 2 * ( Width - 100 ) / 4, 3 * ( Height - 100 ) / 4, 0.5, 1.5 );

  InitNorm( 3 * ( Width - 100 ) / 4, ( Height - 100 ) / 4, 0.5, 1.5 );
  InitNorm( 3 * ( Width - 100 ) / 4, 2 * ( Height - 100 ) / 4, 0.5, 1.5 );
  InitNorm( 3 * ( Width - 100 ) / 4, 3 * ( Height - 100 ) / 4, 0.5, 1.5 );*/

  /*SetCurrent( 1 );

  InitNorm( Width / 2, Height / 2, 5.0, 7.0 );*/
}

void XLifeApplication :: DrawContent( void )
{
  unsigned int PointsNumber;

  PointsNumber = pXLifeField->Fill( pPoints );

  XClearWindow( display, win );

  XDrawPoints( display, win, gc, pPoints, PointsNumber, CoordModeOrigin );
}

Window XLifeApplication :: CreateWindow( void )
{
  Window win;
  int    ScreenNumber;
  int    x;
  int    y;
  int    BorderWidth;
  
  ScreenNumber = DefaultScreen( display );

  if( pModule->IfWidth() ) {
    FieldWidth = pModule->GetWidth();
  }
  else {
    FieldWidth = DisplayWidth( display, ScreenNumber ) - pModule->GetBorderField();
  }

  if( pModule->IfHeight() ) {
    FieldHeight = pModule->GetHeight();
  }
  else {
    FieldHeight = DisplayHeight( display, ScreenNumber ) - pModule->GetBorderField();
  }

  x = 0;
  y = 0;
  
  BorderWidth = 4;

  win = XCreateSimpleWindow( display, RootWindow( display, ScreenNumber ),
                             x, y, FieldWidth, FieldHeight, BorderWidth,
                             BlackPixel( display, ScreenNumber ),
                             WhitePixel( display, ScreenNumber ) );                      

  XSelectInput( display, win, ExposureMask | KeyPressMask |
                ButtonPressMask | StructureNotifyMask );

  return( win );  
}

int XLifeApplication :: GetGC( void )
{
  int           ScreenNumber;
  int           RetCode;
  
  RetCode = 0;

  gc = XCreateGC( display, win, 0, 0 );

  ScreenNumber = DefaultScreen( display );

  XSetForeground( display, gc, BlackPixel( display, ScreenNumber ) );

  return( RetCode );  
}

const int BUF_SIZE = 4;

/* just get a keysym (one of eg XK_... in keysymdef.h) x_lat must have space for four bytes */
static KeySym key_sym_xlat (XEvent * e, char *x_lat)
{
    KeySym key;
    if (e->type == KeyPress || e->type == KeyRelease)
    {
      XLookupString (&(e->xkey), x_lat, BUF_SIZE, &key, NULL);
      return key;
    }
    else
      return 0;
}

KeySym CKeySym( XEvent * e )
{
  char xlat[ BUF_SIZE ];

  return key_sym_xlat (e, xlat);
}

int XLifeApplication :: ProcessEvents( void )
{
  XEvent report;
  int    ExitFlag = 0;
  int    Delay    = 0;
  int    DELTA    = pModule->GetDelta();
  int    RetCode  = 0;

  while( !ExitFlag ) {
    XNextEvent( display, &report );

    switch( report.type ) {
      case Expose:
        if( !report.xexpose.count ) {
          DrawContent();
        }
      break;
      case ClientMessage:
        if( static_cast<unsigned int>( report.xclient.data.l[0] ) == wm_delete_window ) {
          ExitFlag = 1;
        }
      break;
      case KeyPress:
        switch( CKeySym( &report ) ) {
          case XK_q:
            ExitFlag = 1;
          break;
          case XK_s:
            for(;;) {
              pXLifeField->Step();
              DrawContent();
              usleep( Delay );

              if( XCheckMaskEvent( display, KeyPressMask, &report ) ) {
                KeySym k = CKeySym( &report );

                if( k == XK_s ) {
                  break;
                }

                if( k == XK_q ) {
                  ExitFlag = 1;
                  break;
                }

                switch( k ) {
                  case XK_Down:
                    Delay = Delay + DELTA;
                  break;
                  case XK_Up:
                    if( Delay >= DELTA ) {
                      Delay = Delay - DELTA;
                    }
                  break;
                  default:;
                }
              }
            }
          break;
          default:
          break;
        }
      break;

      default:
      break;
    }
  }
    
  return( RetCode );
}

int XLifeApplication :: RunApplication( void )
{
  int    RetCode;
  
  RetCode = 0;

  win = CreateWindow();

  pXLifeField = new XLifeField( FieldWidth, FieldHeight );

  pPoints = new XPoint[ FieldWidth * FieldHeight ];

  pXLifeField->Init( pModule->IfInitRandom(), pModule->GetInitRandom() ); 

  if( win ) {
    RetCode = GetGC();

    if( !RetCode ) {
      XMapWindow( display, win );

      wm_delete_window = XInternAtom( display, "WM_DELETE_WINDOW", False );

      XSetWMProtocols( display, win, &wm_delete_window, 1 );
  
      RetCode = ProcessEvents();
      
      XFreeGC( display, gc );
    }
    else {
      RetCode = 2;
    }   
  }
  else {
    RetCode = 1;
  }  

  return( RetCode );
}

int XLifeApplication :: Run( void )
{
  char    * DisplayName;
  int       RetCode = XLIFE_NO_ERROR;
  
  DisplayName = 0;

  display = XOpenDisplay( DisplayName );

  if( display ) {
    RetCode = RunApplication();

    XCloseDisplay( display );
  }
  else {
    pModule->InfoMessage( XLIFE_OPEN_DISPLAY );
    RetCode = XLIFE_OPEN_DISPLAY;
  }

  return( RetCode );
}

XLifeApplication :: XLifeApplication( XLifeSubModule * pXLifeSubModule ) :
                                      display( 0 ), pXLifeField( 0 ), pPoints( 0 )
{
  pModule = new XLifeModule( pXLifeSubModule );
}

XLifeApplication :: ~XLifeApplication( void )
{
  if( pXLifeField ) {
    delete pXLifeField;
  }

  if( pPoints ) {
    delete pPoints;
  }

  delete pModule;
}

}}

//---------------------------- End of file ---------------------------
