//------------------------------------------------------------------------------
//
// File:    xlifeProg.h
//
// Purpose: Life game for XWindow
//   
// (c) Plekhanov Andrey
//
// Initial version 0.1 03.01.2005
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//------------------------------------------------------------------------------

#ifndef 	_XLIFE_H_
#define         _XLIFE_H_	1

#include <X11/Xlib.h>

namespace aps {

  namespace xlifeappl {

class XLifeModule;
class XLifeSubModule;

typedef unsigned short POINT_VALUE;

const short POINT1 = 0x0001;
const short POINT2 = 0x0002;
const short POINT3 = 0x0004;
const short POINT4 = 0x0008;
const short POINT5 = 0x0010;
const short POINT6 = 0x0020;
const short POINT7 = 0x0040;
const short POINT8 = 0x0080;
const short POINT9 = 0x0100;

class OnePoint
{
  private:

    POINT_VALUE * pValue;

    POINT_VALUE * ValueArray[ 9 ];

  public:

    OnePoint( POINT_VALUE * apValue );

    virtual ~OnePoint( void );

    void SetValueArray( const unsigned int Number, const unsigned int Width,
                        const unsigned int Height, POINT_VALUE * pField );

    void Set( void ) {
      *(ValueArray[ 0 ]) |= POINT9;
      *(ValueArray[ 1 ]) |= POINT8;
      *(ValueArray[ 2 ]) |= POINT7;
      *(ValueArray[ 3 ]) |= POINT6;
      *(ValueArray[ 4 ]) |= POINT5;
      *(ValueArray[ 5 ]) |= POINT4;
      *(ValueArray[ 6 ]) |= POINT3;
      *(ValueArray[ 7 ]) |= POINT2;
      *(ValueArray[ 8 ]) |= POINT1;
    }
};

class XLifeField
{
  private:
    int          page;

    unsigned int Width;
    unsigned int Height;
    unsigned int PointsNumber;
    unsigned int Table[ 512 ];

    OnePoint ** ppPoint1;
    OnePoint ** ppPoint2;

    POINT_VALUE * pField1;
    POINT_VALUE * pField2;

    int Sum( const POINT_VALUE Value ) const;

  public:

    XLifeField( const unsigned int aWidth, const unsigned int aHeight );

    virtual ~XLifeField( void );

    void Step( void );

    unsigned int Fill( XPoint * pPoints ) const;

    void InitRec( const unsigned int StartX, const unsigned int StartY,
                  const unsigned int StartWidth, const unsigned int StartHeight );

    void InitNorm( const unsigned int StartX, const unsigned int StartY,
                   const double Sigma, const double Number );

    void Init( const bool IfInitRandom, const unsigned int InitRandom );
};

class XLifeApplication
{
  private:

    XLifeModule  * pModule;
    Display      * display;
    Window         win;
    GC             gc;
    Atom           wm_delete_window;
    unsigned int   FieldWidth;
    unsigned int   FieldHeight;
    XLifeField   * pXLifeField;
    XPoint       * pPoints;

    void DrawContent( void );

    Window CreateWindow( void );

    int GetGC( void );

    int ProcessEvents( void );

    int RunApplication( void );

  public:

    XLifeApplication( XLifeSubModule * pXLifeSubModule );

    virtual ~XLifeApplication( void );

    int Run( void );
};

}}

#endif

//---------------------------- End of file ---------------------------


